package com.common.integrate.video

import android.content.Context
import android.media.MediaMetadataRetriever
import com.common.bean.VideoRes
import com.common.inter.ICompressProgress
import com.hw.videoprocessor.VideoProcessor
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.SDCardUtils
import java.io.File

/**
 * @date： Crate by wangjunwei on 2022/7/26
 * @description：视频处理管理类：本地视频压缩、裁剪、播放等等
 **/
class VideoProcessManager {

    companion object {

        /***
         * 压缩视频参数，高中低，低于多少不压缩
         */

        private const val COMPRESS_SOURCE = 1

        private const val COMPRESS_HIGH = 0.75
        private const val COMPRESS_MIDDLE = 0.5

        private const val COMPRESS_MIDDLE_LOW = 0.35

        private const val COMPRESS_LOW = 0.25

        /***
         * 低于多少M不压缩分辨率
         */
        private  const val COMPRESS_SMALLEST_SIZE = 10 * 1024 * 1024

        /***
         * 低于多少宽高不压缩分辨率
         */
        private  const val COMPRESS_SMALLEST_WIDTH = 720
        private  const val COMPRESS_SMALLEST_HEIGHT = 1280


        /***
         * 记录执行压缩的回调序列，方便管理：取消等
         */
        private val compressProgressMap = mutableMapOf<Int, ICompressProgress>()

        /***
         * 当前正在执行的回调tag
         */
        private var currentCompressProgressListenerTag: Int? = null

        /***
         * 压缩视频，质量高：大小压缩到原先的 0.75 * 0.75
         */
        fun compressVideoHigh(
            context: Context?,
            videoRes: VideoRes?,
            iCompressProgress: ICompressProgress,
            compressProgressListenerTag: Int
        ) {
            compressVideo(context, videoRes, iCompressProgress, COMPRESS_HIGH, compressProgressListenerTag)
        }


        /***
         * 压缩视频，质量中：大小压缩到原先的 1 / 4
         */
        fun compressVideoMiddle(
            context: Context?,
            videoRes: VideoRes?,
            iCompressProgress: ICompressProgress,
            compressProgressListenerTag: Int
        ) {
            compressVideo(context, videoRes, iCompressProgress, COMPRESS_MIDDLE, compressProgressListenerTag)
        }


        /***
         * 压缩视频，质量中低：大小压缩到原先的 1 / 8
         */
        fun compressVideoMiddleLow(
            context: Context?,
            videoRes: VideoRes?,
            iCompressProgress: ICompressProgress,
            compressProgressListenerTag: Int
        ) {
            compressVideo(context, videoRes, iCompressProgress, COMPRESS_MIDDLE_LOW, compressProgressListenerTag)
        }


        /***
         * 压缩视频，质量低：大小压缩到原先的 1 / 8
         */
        fun compressVideoLow(
            context: Context?,
            videoRes: VideoRes?,
            iCompressProgress: ICompressProgress,
            compressProgressListenerTag: Int
        ) {
            compressVideo(context, videoRes, iCompressProgress, COMPRESS_LOW, compressProgressListenerTag)
        }

        /***
         * 压缩视频
         * @param compressProgressListenerTag 回调对应的tag
         */
        private fun compressVideo(
            context: Context?,
            videoRes: VideoRes?,
            iCompressProgress: ICompressProgress,
            compressScale: Double,
            compressProgressListenerTag: Int) {

            //只保留一次回调记录，开启新的回调，取消上一次的回调
            currentCompressProgressListenerTag?.let {
                val iCompressProgress = compressProgressMap[it]
                iCompressProgress?.onCancelCompress()
                compressProgressMap.remove(it)
            }
            currentCompressProgressListenerTag = compressProgressListenerTag
            compressProgressMap[compressProgressListenerTag] = iCompressProgress

            context?.let { itContext ->
                videoRes?.let { itVideo ->
                    ThreadUtil.io {
                        //目标文件路径
                        val destFile =
                            File(SDCardUtils.getInternalStorageMovieDir(itContext), itVideo.name)
                        //if (destFile.exists()){
                        //    destFile.delete()
                        //}
                        val retriever = MediaMetadataRetriever()
                        retriever.setDataSource(itContext, itVideo.uri)
                        val originWidth =
                            retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_WIDTH)
                                ?.toInt()
                        val originHeight =
                            retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_VIDEO_HEIGHT)
                                ?.toInt()
                        val bitrate =
                            retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_BITRATE)
                                ?.toInt() ?: VideoProcessor.DEFAULT_FRAME_RATE

                        //调整压缩系数
                        var compressScaleReal: Double = compressScale
                        //1 大小小于10M不做分辨率压缩
                        if (COMPRESS_SMALLEST_SIZE >= itVideo.size) {
                            compressScaleReal = COMPRESS_SOURCE.toDouble()
                        }
                        //2 宽高小于720 1080不做分辨率压缩
                        if (originWidth ?: 0 <= COMPRESS_SMALLEST_WIDTH || originHeight?: 0 <= COMPRESS_SMALLEST_HEIGHT){
                            compressScaleReal =  COMPRESS_SOURCE.toDouble()
                        }
                        //目标宽高
                        val destWidth = (originWidth ?: 0) * compressScaleReal
                        val destHeight = (originHeight ?:0) * compressScaleReal

                        VideoProcessor
                            .processor(itContext)
                            .input(itVideo.uri)
                            .output(destFile.path)
                            .outWidth(destWidth.toInt())
                            .outHeight(destHeight.toInt())
                            .bitrate(bitrate / 6)
                            .frameRate(60)
                            .progressListener { progress ->
                                val progressInt = (progress * 100).toInt()
                                val iCompressProgress = compressProgressMap[compressProgressListenerTag]
                                    ?: return@progressListener
                                //延迟1s，解决少部分机型压缩进度100%，压缩文件还没完全生成的问题，导致上传的视频不可播放问题
                                ThreadUtil.uiDelay ({
                                    iCompressProgress?.compressProgress(progressInt)
                                    if (progressInt == 100) {
                                        compressProgressMap?.remove(compressProgressListenerTag)
                                        iCompressProgress?.compressComplete(
                                            destFile.path,
                                            videoRes?.size,
                                            destFile.length()
                                        )
                                    }
                                }, 1000)
                            }.process()
                    }
                }
            }

        }

        /**
         * 取消当前压缩的回调
         */
        fun cancelCompressProcess(){
            currentCompressProgressListenerTag?.let {
                val iCompressProgress = compressProgressMap[it]
                iCompressProgress?.onCancelCompress()
                compressProgressMap.remove(it)
            }
        }
    }

}