package com.lagou.lagouee.widget

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.ValueAnimator
import android.content.Context
import androidx.core.content.ContextCompat
import android.text.Spannable
import android.text.SpannableStringBuilder
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.FrameLayout
import com.alpha.lagouapk.R
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.definition.b.BAid
import com.lagou.chat.chatlibrary.util.ChatDateUtils
import com.lagou.lagouee.ui.dialog.ResumeAlertDialog
import com.lagou.lagouee.ui.resume.NewResumeRecord
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.lagouee.utils.SPUtilsB
import kotlinx.android.synthetic.main.layout_reusme_coming.view.*
import kotlin.math.abs

/**
 * 新简历到来的通知View
 *
 * @author yph
 * @date 2019/9/9
 */
class ResumeComingView(context: Context, attrs: AttributeSet? = null) : FrameLayout(context, attrs), NewResumeRecord.OnRecordChangedListener {
    private val mSpeed: Float
    private val mEllipsisWidth: Float
    private val mGreen: Int
    private val mBlack: Int
    private var mOtherWidth: Int = 0
    private var mTextMaxWidth: Int = 0

    private val mAnimator: ValueAnimator by lazy(LazyThreadSafetyMode.NONE) {
        val animator = ValueAnimator()
        animator.addUpdateListener {
            translationX = it.animatedValue as Float
        }
        animator
    }

    private val mAutoHideListener: AnimatorListenerAdapter by lazy(LazyThreadSafetyMode.NONE) {
        val listener = object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator?) {
                prepareAutoHide()
            }
        }
        listener
    }

    private val mHidingState: Runnable = Runnable { animateTranslation(this@ResumeComingView.tag as Int) }

    init {
        LayoutInflater.from(context).inflate(R.layout.layout_reusme_coming, this)
        mGreen = ContextCompat.getColor(context, R.color.theme_text_color)
        mBlack = ContextCompat.getColor(context, R.color.color_2A2A2A)
        val max = resources.displayMetrics.widthPixels - resources.getDimensionPixelSize(R.dimen.dp_62)
        mSpeed = max / 500.0f

        tv_resume_coming_tip.apply {
            mEllipsisWidth = paint.measureText(ELLIPSIS)
            setOnClickListener {
                // 文本点击
                IntentUtils.startResumeNotifyManagerActivity(context, null)
                // 关闭点击 右侧收回
                mHidingState.run()
                AnalyticHelper.getInstance().sendClickInfo(BAid._1nh3)
            }
        }

        iv_resume_coming_close.setOnClickListener {
            // 弹窗检查
            NewResumeRecord.getInstance().increaseCloseCount()
            if (NewResumeRecord.getInstance().closeCount >= 3) {
                showRejectDialog()
            }
            // 关闭点击 右侧收回
            mHidingState.run()
            AnalyticHelper.getInstance().sendClickInfo(BAid._1nh4)
        }

        post {
            mOtherWidth = resources.getDimensionPixelSize(R.dimen.dp_20) + iv_resume_coming_close.width
            mTextMaxWidth = max - mOtherWidth
            tv_resume_coming_tip.maxWidth = mTextMaxWidth
            tag = width
            // 初始化隐藏
            translationX = width.toFloat()
        }
    }

    /**
     * 展示动画
     */
    fun show(tip: CharSequence, wholeWidth: Float) {
        tv_resume_coming_tip?.text = tip
        post {
            // 更新tag
            tag = wholeWidth.toInt()
            val curr = translationX.toInt()
            if (curr == 0) {
                // 已完全展示
                prepareAutoHide()
            } else {
                // 移动到完全展示
                animateTranslation(0)
            }
        }
    }

    /**
     * 自动隐藏
     */
    private fun prepareAutoHide() {
        clear()
        postDelayed(mHidingState, DELAY_MILLIS)
    }

    private fun animateTranslation(dest: Int) {
        clear()
        val curr = translationX.toInt()
        if (curr == dest) {
            if (curr == 0) {
                // 完全展示
                postDelayed(mHidingState, DELAY_MILLIS)
            }
            return
        }

        if (dest == 0) {
            mAnimator.addListener(mAutoHideListener)
        }
        mAnimator.setFloatValues(curr.toFloat(), dest.toFloat())
        val duration = (abs(curr - dest) / mSpeed + 0.5f).toLong()
        mAnimator.duration = duration
        mAnimator.start()
    }

    private fun clear() {
        removeCallbacks(mHidingState)
        // cancel会引发onAnimationEnd调用，故先进行阻断
        mAnimator.removeListener(mAutoHideListener)
        if (mAnimator.isStarted || mAnimator.isRunning) {
            mAnimator.cancel()
        }
    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        NewResumeRecord.getInstance().addOnRecordChangedListener(this)
    }

    override fun onRecordRemoved() {
        // empty
    }

    override fun onRecordChanged(positionId: Int) {
        // empty
    }

    override fun onRecordAdded() {
        if (ChatDateUtils.isSameDay(System.currentTimeMillis(), SPUtilsB.getCloseResumeMillis())) {
            // 今天不能再提醒
            return
        }
        val positionCount = NewResumeRecord.getInstance().size()
        val resumeCount = NewResumeRecord.getInstance().resumeCount
        if (positionCount <= 0 || resumeCount <= 0) {
            return
        }
        val lastPositionName = NewResumeRecord.getInstance().lastPositionName
        // 检查lastPositionName是否需要被截断
        val tip = when (positionCount) {
            1 -> "${lastPositionName}收到${resumeCount}封新简历"
            else -> "${lastPositionName}等${positionCount}个职位收到${resumeCount}封新简历"
        }

        val paint = tv_resume_coming_tip.paint
        // 标记需要截断的长度
        var length = -1
        // 文本原长度
        var cacheWidth = paint.measureText(tip)
        if (cacheWidth > mTextMaxWidth) {
            // 截断
            val endStr = tip.substring(lastPositionName.length)
            val startWidth = mTextMaxWidth - mEllipsisWidth - paint.measureText(endStr)
            length = paint.breakText(lastPositionName, true, startWidth, null)
            // 调整后的文本长度
            cacheWidth = mTextMaxWidth.toFloat()
        }
        // 整个View长度
        cacheWidth += mOtherWidth
        val str1 = if (length > 0) lastPositionName.substring(0, length) else lastPositionName
        when (positionCount) {
            1 -> {
                val str2 = if (length > 0) "${ELLIPSIS}收到" else "收到"
                show(make(str1, str2, resumeCount.toString()), cacheWidth)
            }
            else -> {
                val str2 = if (length > 0) "${ELLIPSIS}等${positionCount}个职位收到" else "等${positionCount}个职位收到"
                show(make(str1, str2, resumeCount.toString()), cacheWidth)
            }
        }

        AnalyticHelper.getInstance().sendClickInfo(BAid._1nh2)
    }

    private fun make(str1: String, str2: String, str3: String, str4: String = "封新简历"): SpannableStringBuilder {
        val builder = SpannableStringBuilder()
        var cache = builder.length
        builder.append(str1)
        builder.setSpan(ForegroundColorSpan(mGreen), cache, builder.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        cache = builder.length
        builder.append(str2)
        builder.setSpan(ForegroundColorSpan(mBlack), cache, builder.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        cache = builder.length
        builder.append(str3)
        builder.setSpan(ForegroundColorSpan(mGreen), cache, builder.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        cache = builder.length
        builder.append(str4)
        builder.setSpan(ForegroundColorSpan(mBlack), cache, builder.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        return builder
    }

    /**
     * 不想再接收此类提醒
     */
    private fun showRejectDialog() {
        val a = context as? BaseActivity ?: return
        if (a.isActivityStateEnable) {
            ResumeAlertDialog().show(a.supportFragmentManager)
        }
    }

    override fun onDetachedFromWindow() {
        NewResumeRecord.getInstance().removeOnRecordChangedListener(this)
        clear()
        translationX = width.toFloat()
        super.onDetachedFromWindow()
    }

    companion object {
        private const val DELAY_MILLIS: Long = 2000

        private const val ELLIPSIS: String = "\u2026"
    }
}
